@@ -28,7 +28,6 @@
 #ifndef MDDS_MULTI_TYPE_VECTOR_HPP
 #define MDDS_MULTI_TYPE_VECTOR_HPP
 
-#include "default_deleter.hpp"
 #include "global.hpp"
 #include "multi_type_vector_types.hpp"
 #include "multi_type_vector_itr.hpp"
@@ -130,8 +129,14 @@ private:
 
         block();
         block(size_type _size);
+        block(size_type _size, element_block_type* _data);
         block(const block& other);
+        block(block&& other);
         ~block();
+        void swap(block& other);
+        void clone_to(block& other) const;
+
+        block& operator=(block);
     };
 
     struct element_block_deleter : public std::unary_function<void, const element_block_type*>
@@ -142,7 +147,7 @@ private:
         }
     };
 
-    typedef std::vector<block*> blocks_type;
+    typedef std::vector<block> blocks_type;
 
     struct blocks_to_transfer
     {
@@ -1056,25 +1061,19 @@ private:
     /**
      * Delete only the element block owned by an outer block.
      *
-     * @param p pointer to the instance of an outer block that may own an
+     * @param blk reference to the instance of an outer block that may own an
      *          element block instance.
      */
-    void delete_element_block(block* p);
+    void delete_element_block(block& blk);
 
     /**
-     * Delete an outer block.
-     *
-     * @param p pointer to the instance of an outer block to delete.
-     */
-    void delete_block(block* p);
-
-    /**
-     * Delete one or more outer blocks in the specified iterator ranges.
+     * Delete the element block(s) owned by one or more outer blocks in the
+     * specified iterator ranges.
      *
      * @param it start position.
      * @param it_end end position (not inclusive).
      */
-    void delete_blocks(typename blocks_type::iterator it, typename blocks_type::iterator it_end);
+    void delete_element_blocks(typename blocks_type::iterator it, typename blocks_type::iterator it_end);
 
     template<typename _T>
     iterator set_impl(size_type pos, size_type start_row, size_type block_index, const _T& value);
@@ -1276,9 +1275,9 @@ private:
      * @param new_block_size size of the new block
      * @param overwrite whether or not to overwrite the elements replaced by
      *                  the new block.
-     * @return pointer to the middle block
+     * @return reference to the middle block
      */
-    block* set_new_block_to_middle(
+    block& set_new_block_to_middle(
         size_type block_index, size_type offset, size_type new_block_size, bool overwrite);
 
     block* get_previous_block_of_type(size_type block_index, element_category_type cat);
